package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.SpaceOptimizeShortDurationEnt;
import pps.core.space.entity.SpaceOptimizeShortWellheadEnt;
import pps.core.space.mapper.SpaceOptimizeShortDurationMapper;
import pps.core.space.mapper.SpaceOptimizeShortWellheadMapper;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadInput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 间开优化对外提供接口服务
 *
 * @author ZWT
 * @date 2024/04/08 14:21
 */
@XService
@XText("间开优化对外提供接口服务")
public class SpaceOptimizeApiService {

    /**
     * 每日间开优化结果
     *
     * @param context 上下文
     * @return {@link XListResult}<{@link QuerySpaceOptimizeShortWellheadOutput}>
     */
    @XApiPost(anonymous = true)
    @XText("极短期间开优化--每日间开优化结果")
    public XListResult<QuerySpaceOptimizeShortWellheadOutput> getDailyOptimization(XContext context, QuerySpaceOptimizeShortWellheadInput input) {
        Date queryDate = input.getQueryDate();
        if (Objects.isNull(queryDate)) {
            queryDate = DateUtil.date();
        }
        DateTime beginOfDay = DateUtil.beginOfDay(queryDate);
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .eq(SpaceOptimizeShortWellheadEnt::getOptimizeDate, beginOfDay)
                .orderByAsc(SpaceOptimizeShortWellheadEnt::getStartSeq)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(Collections.emptyList());
        }
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, beginOfDay)
                .in(SpaceOptimizeShortDurationEnt::getWellheadId, wellheadList.stream()
                        .map(SpaceOptimizeShortWellheadEnt::getWellheadId)
                        .collect(Collectors.toList())
                )
                .orderByAsc(SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
        List<QuerySpaceOptimizeShortWellheadOutput> outputs = new ArrayList<>(wellheadList.size());
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (SpaceOptimizeShortWellheadEnt wellheadEnt : wellheadList) {
            outputs.add(
                    QuerySpaceOptimizeShortWellheadOutput.builder()
                            .wellNumber(wellheadEnt.getWellNumber())
                            .optimizeList(ServiceUtil.getAllDateDuration(collect.get(wellheadEnt.getWellheadId())))
                            .build()
            );
        }
        return XListResult.success(outputs);
    }
}
